#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
wix: a SAT frontend
Copyright (C) 2009, 2010  Jérôme Poisson (goffi@goffi.org)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""



import wx
import pdb
from xml.dom import minidom
from logging import debug, info, warning, error
from tools.jid  import JID


class Form(wx.Frame):
    """Create a form from a SàT xml"""

    def __init__(self, host, xml_data='', title="Form", options=[], misc={}):
        super(Form, self).__init__(None, title=title)

        self.host = host
        self.options = options
        self.misc = misc
        self.ctl_list = []  # usefull to access ctrl

        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.SetSizer(self.sizer)
        self.SetAutoLayout(True)
        
        #events
        if not 'NO_CANCEL' in self.options:
            self.Bind(wx.EVT_CLOSE, self.onClose, self)
        
        self.MakeModal()

        self.constructForm(xml_data)

        self.Show()

    def constructForm(self, xml_data):
        panel=wx.Panel(self)
        panel.sizer = wx.BoxSizer(wx.VERTICAL)

        cat_dom = minidom.parseString(xml_data.encode('utf-8'))
        
        for elem in cat_dom.documentElement.getElementsByTagName("elem"):
            name = elem.getAttribute("name")
            label = elem.getAttribute("label") if elem.hasAttribute('label') else name
            type = elem.getAttribute("type")
            value = elem.getAttribute("value") if elem.hasAttribute('value') else u''
            sizer = wx.BoxSizer(wx.HORIZONTAL)
            if type=="text":
                ctrl = wx.StaticText(panel, -1, value)
            elif type=="string":
                label=wx.StaticText(panel, -1, label+": ")
                ctrl = wx.TextCtrl(panel, -1, value)
                self.ctl_list.append({'name':name, 'type':type, 'control':ctrl})
                sizer.Add(label)
            elif type=="password":
                label=wx.StaticText(panel, -1, label+": ")
                ctrl = wx.TextCtrl(panel, -1, value, style=wx.TE_PASSWORD)
                self.ctl_list.append({'name':name, 'type':type, 'control':ctrl})
                sizer.Add(label)
            elif type=="list":
                label=wx.StaticText(panel, -1, label+": ")
                ctrl = wx.ListBox(panel, -1, choices=[option.getAttribute("value") for option in elem.getElementsByTagName("option")], style=wx.LB_SINGLE)
                self.ctl_list.append({'name':name, 'type':type, 'control':ctrl})
            else:
                error(_("FIXME FIXME FIXME: type [%s] is not implemented") % type)  #FIXME !
                raise NotImplementedError
            sizer.Add(ctrl, 1, flag=wx.EXPAND)
            #self.ctl_list[(name, category)] = ctrl
            panel.sizer.Add(sizer, flag=wx.EXPAND)

        dialogButtons = wx.StdDialogButtonSizer()
        submitButton = wx.Button(panel,wx.ID_OK, label=_("Submit"))
        dialogButtons.AddButton(submitButton)
        panel.Bind(wx.EVT_BUTTON, self.onFormSubmitted, submitButton)
        if not 'NO_CANCEL' in self.options:
            cancelButton = wx.Button(panel,wx.ID_CANCEL)
            dialogButtons.AddButton(cancelButton)
            panel.Bind(wx.EVT_BUTTON, self.onFormCancelled, cancelButton)
        dialogButtons.Realize()
        panel.sizer.Add(dialogButtons, flag=wx.ALIGN_CENTER_HORIZONTAL)

        panel.SetSizer(panel.sizer)
        panel.SetAutoLayout(True)
        panel.sizer.Fit(self)
        self.sizer.Add(panel, 1, flag=wx.EXPAND)
        cat_dom.unlink()

    def onFormSubmitted(self, event):
        """Called when submit button is clicked"""
        debug(_("Submitting form"))
        data = []
        for ctrl in self.ctl_list:
            if isinstance(ctrl['control'], wx.ListBox):
                data.append((ctrl['name'], ctrl['control'].GetStringSelection()))
            else:
                data.append((ctrl["name"], ctrl["control"].GetValue()))
        if self.misc.has_key('action_back'): #FIXME FIXME FIXME: WTF ! Must be cleaned
            id = self.misc['action_back']("SUBMIT",self.misc['target'], data)
            self.host.current_action_ids.add(id)
        elif self.misc.has_key('callback'):
            self.misc['callback'](data)
        else:
            warning (_("The form data is not sent back, the type is not managed properly"))
        self.MakeModal(False)
        self.Destroy()
        
    def onFormCancelled(self, event):
        """Called when cancel button is clicked"""
        debug(_("Cancelling form"))
        self.MakeModal(False)
        self.Close()
   
    def onClose(self, event):
        """Close event: we have to send the form."""
        debug(_("close"))
        self.MakeModal(False)
        event.Skip()

